@@ -71,6 +71,8 @@ IconViewIndexWidget::IconViewIndexWidget(const IconViewDelegate *delegate, const
     m_delegate->getView()->m_editValid = false;
     m_delegate->getView()->m_renameTimer->start();
 
+    m_is_dragging = m_delegate->getView()->isDraggingState();
+
     QSize size = delegate->sizeHint(option, index);
     setMinimumSize(size);
 
@@ -104,8 +106,21 @@ IconViewIndexWidget::IconViewIndexWidget(const IconViewDelegate *delegate, const
     adjustPos();
 
     auto textSize = IconViewTextHelper::getTextSizeForIndex(opt, index, 2);
-
     int fixedHeight = 5 + iconExpectedSize.height() + 5 + textSize.height() + 5;
+
+    int y_bottom = option.rect.y() + fixedHeight + 20;
+    //qDebug() << "Y:" <<option.rect.y() <<fixedHeight <<m_delegate->getView()->height();
+    b_elide_text = false;
+    if ( y_bottom > m_delegate->getView()->height() && opt.text.length() > ELIDE_TEXT_LENGTH)
+    {
+        b_elide_text = true;
+        int  charWidth = opt.fontMetrics.averageCharWidth();
+        opt.text = opt.fontMetrics.elidedText(opt.text, Qt::ElideRight, ELIDE_TEXT_LENGTH * charWidth);
+        //recount size
+        textSize = IconViewTextHelper::getTextSizeForIndex(opt, index, 2);
+        fixedHeight = 5 + iconExpectedSize.height() + 5 + textSize.height() + 5;
+    }
+
     if (fixedHeight >= option.rect.height())
         setFixedHeight(fixedHeight);
     else
@@ -151,6 +166,11 @@ void IconViewIndexWidget::paintEvent(QPaintEvent *e)
     auto tmp = opt.text;
     opt.text = nullptr;
     QApplication::style()->drawControl(QStyle::CE_ItemViewItem, &opt, &p, opt.widget);
+    if (b_elide_text)
+    {
+        int  charWidth = opt.fontMetrics.averageCharWidth();
+        tmp = opt.fontMetrics.elidedText(tmp, Qt::ElideRight, ELIDE_TEXT_LENGTH * charWidth);
+    }
     opt.text = std::move(tmp);
 
     //auto textRectF = QRectF(0, m_delegate->getView()->iconSize().height(), this->width(), this->height());
@@ -158,7 +178,7 @@ void IconViewIndexWidget::paintEvent(QPaintEvent *e)
 
     p.setPen(opt.palette.highlightedText().color());
     p.translate(0, m_delegate->getView()->iconSize().height() + 5);
-    IconViewTextHelper::paintText(&p, m_option, m_index, 9999, 2, 0);
+    IconViewTextHelper::paintText(&p, opt, m_index, 9999, 2, 0);
 
 //    p.translate(-1, m_delegate->getView()->iconSize().height() + 13);
 //    //m_edit->document()->drawContents(&p);
@@ -174,6 +194,21 @@ void IconViewIndexWidget::paintEvent(QPaintEvent *e)
         return;
     }
     auto info = m_info.lock();
+
+    // draw color symbols
+    auto colors = info->getColors();
+    int offset = 0;
+    for (auto color : colors) {
+        p.save();
+        p.setRenderHint(QPainter::Antialiasing);
+        p.translate(2, 2);
+        p.setPen(opt.palette.highlightedText().color());
+        p.setBrush(color);
+        p.drawEllipse(QRectF(offset, 0, 10, 10));
+        p.restore();
+        offset += 10;
+    }
+
     //paint symbolic link emblems
     if (info->isSymbolLink()) {
         QIcon icon = QIcon::fromTheme("emblem-symbolic-link");
@@ -201,6 +236,13 @@ void IconViewIndexWidget::mousePressEvent(QMouseEvent *e)
 {
     if (e->button() == Qt::LeftButton) {
         IconView *view = m_delegate->getView();
+
+        if (view->isDraggingState() || m_is_dragging) {
+            view->m_renameTimer->stop();
+            view->m_editValid = false;
+            return QWidget::mousePressEvent(e);
+        }
+
         view->m_editValid = true;
         if (view->m_renameTimer->isActive()) {
             if (view->m_renameTimer->remainingTime() < 2250 && view->m_renameTimer->remainingTime() > 0) {
@@ -225,6 +267,18 @@ void IconViewIndexWidget::mousePressEvent(QMouseEvent *e)
     QWidget::mousePressEvent(e);
 }
 
+void IconViewIndexWidget::mouseMoveEvent(QMouseEvent *e)
+{
+    QWidget::mouseMoveEvent(e);
+    m_is_dragging = true;
+}
+
+void IconViewIndexWidget::mouseReleaseEvent(QMouseEvent *e)
+{
+    QWidget::mouseReleaseEvent(e);
+    m_is_dragging = false;
+}
+
 void IconViewIndexWidget::mouseDoubleClickEvent(QMouseEvent *event)
 {
     m_delegate->getView()->doubleClicked(m_index);
@@ -237,6 +291,11 @@ void IconViewIndexWidget::adjustPos()
     if (m_index.model() != view->model())
         return;
 
+    if (!view->selectionModel()->selectedIndexes().contains(m_index)) {
+        this->close();
+        return;
+    }
+
     auto visualRect = view->visualRect(m_index);
     if (this->mapToParent(QPoint()) != visualRect.topLeft())
         this->move(visualRect.topLeft());
