@@ -1,7 +1,7 @@
 /*
  * Peony-Qt's Library
  *
- * Copyright (C) 2019, Tianjin KYLIN Information Technology Co., Ltd.
+ * Copyright (C) 2020, KylinSoft Co., Ltd.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -39,6 +39,7 @@
 #include "file-operation-manager.h" //FileOpInfo
 
 #include "file-utils.h"
+#include "bookmark-manager.h"
 
 #include "volume-manager.h"
 
@@ -48,6 +49,8 @@
 #include "file-launch-action.h"
 #include "file-lauch-dialog.h"
 
+#include "file-operation-error-dialog.h"
+
 #include <QDesktopServices>
 #include <QUrl>
 #include <QMessageBox>
@@ -58,6 +61,7 @@
 #include <QFileIconProvider>
 
 #include <QLocale>
+#include <QStandardPaths>
 
 #include <QDebug>
 
@@ -74,7 +78,7 @@ DirectoryViewMenu::DirectoryViewMenu(DirectoryViewWidget *directoryView, QWidget
     fillActions();
 }
 
-DirectoryViewMenu::DirectoryViewMenu(FMWindow *window, QWidget *parent) : QMenu(parent)
+DirectoryViewMenu::DirectoryViewMenu(FMWindowIface *window, QWidget *parent) : QMenu(parent)
 {
     m_top_window = window;
     m_view = window->getCurrentPage()->getView();
@@ -97,6 +101,9 @@ void DirectoryViewMenu::fillActions()
     if (m_directory.startsWith("search://"))
         m_is_search = true;
 
+    if (m_directory.startsWith("burn://"))
+        m_is_cd = true;
+
     //add open actions
     auto openActions = constructOpenOpActions();
     if (!openActions.isEmpty())
@@ -150,14 +157,15 @@ const QList<QAction *> DirectoryViewMenu::constructOpenOpActions()
     bool isBackgroundMenu = m_selections.isEmpty();
     if (isBackgroundMenu) {
         l<<addAction(QIcon::fromTheme("window-new-symbolic"), tr("Open in &New Window"));
-        connect(l.last(), &QAction::triggered, [=](){
-            FMWindow *newWindow = new FMWindow(m_directory);
+        connect(l.last(), &QAction::triggered, [=]() {
+            auto windowIface = m_top_window->create(m_directory);
+            auto newWindow = dynamic_cast<QWidget *>(windowIface);
             newWindow->setAttribute(Qt::WA_DeleteOnClose);
             //FIXME: show when prepared?
             newWindow->show();
         });
         l<<addAction(QIcon::fromTheme("tab-new-symbolic"), tr("Open in New &Tab"));
-        connect(l.last(), &QAction::triggered, [=](){
+        connect(l.last(), &QAction::triggered, [=]() {
             if (!m_top_window)
                 return;
             QStringList uris;
@@ -170,31 +178,49 @@ const QList<QAction *> DirectoryViewMenu::constructOpenOpActions()
             auto displayName = info->displayName();
             if (displayName.isEmpty())
                 displayName = FileUtils::getFileDisplayName(info->uri());
+            //when name is too long, show elideText
+            //qDebug() << "charWidth:" <<charWidth;
+            if (displayName.length() > ELIDE_TEXT_LENGTH)
+            {
+                int  charWidth = fontMetrics().averageCharWidth();
+                displayName = fontMetrics().elidedText(displayName, Qt::ElideRight, ELIDE_TEXT_LENGTH * charWidth);
+            }
             if (info->isDir()) {
+                //add to bookmark option
+                l<<addAction(QIcon::fromTheme("bookmark-add-symbolic"), tr("Add to bookmark"));
+                connect(l.last(), &QAction::triggered, [=]() {
+                     //qDebug() <<"add to bookmark:" <<info->uri();
+                     auto bookmark = BookMarkManager::getInstance();
+                     if (bookmark->isLoaded()) {
+                         bookmark->addBookMark(info->uri());
+                     }
+                });
+
                 l<<addAction(QIcon::fromTheme("document-open-symbolic"), tr("&Open \"%1\"").arg(displayName));
-                connect(l.last(), &QAction::triggered, [=](){
+                connect(l.last(), &QAction::triggered, [=]() {
                     if (!m_top_window)
                         return;
                     m_top_window->goToUri(m_selections.first(), true);
                 });
                 l<<addAction(QIcon::fromTheme("window-new-symbolic"), tr("Open \"%1\" in &New Window").arg(displayName));
-                connect(l.last(), &QAction::triggered, [=](){
-                    FMWindow *newWindow = new FMWindow(m_selections.first());
+                connect(l.last(), &QAction::triggered, [=]() {
+                    auto windowIface = m_top_window->create(m_selections.first());
+                    auto newWindow = dynamic_cast<QWidget *>(windowIface);
                     newWindow->setAttribute(Qt::WA_DeleteOnClose);
                     //FIXME: show when prepared?
                     newWindow->show();
                 });
                 l<<addAction(QIcon::fromTheme("tab-new-symbolic"), tr("Open \"%1\" in New &Tab").arg(displayName));
-                connect(l.last(), &QAction::triggered, [=](){
+                connect(l.last(), &QAction::triggered, [=]() {
                     if (!m_top_window)
                         return;
                     m_top_window->addNewTabs(m_selections);
                 });
             } else if (!info->isVolume()) {
                 l<<addAction(QIcon::fromTheme("document-open-symbolic"), tr("&Open \"%1\"").arg(displayName));
-                connect(l.last(), &QAction::triggered, [=](){
+                connect(l.last(), &QAction::triggered, [=]() {
                     auto uri = m_selections.first();
-                    FileLaunchManager::openAsync(uri);
+                    FileLaunchManager::openAsync(uri, false, false);
                 });
                 auto openWithAction = addAction(tr("Open \"%1\" with...").arg(displayName));
                 //FIXME: add sub menu for open with action.
@@ -210,14 +236,14 @@ const QList<QAction *> DirectoryViewMenu::constructOpenOpActions()
                     openWithMenu->addAction(static_cast<QAction*>(action));
                 }
                 openWithMenu->addSeparator();
-                openWithMenu->addAction(tr("&More applications..."), [=](){
+                openWithMenu->addAction(tr("&More applications..."), [=]() {
                     FileLauchDialog d(m_selections.first());
                     d.exec();
                 });
                 openWithAction->setMenu(openWithMenu);
             } else {
                 l<<addAction(tr("&Open"));
-                connect(l.last(), &QAction::triggered, [=](){
+                connect(l.last(), &QAction::triggered, [=]() {
                     auto uri = m_selections.first();
                     //FIXME:
                     m_top_window->goToUri(uri, true);
@@ -225,7 +251,7 @@ const QList<QAction *> DirectoryViewMenu::constructOpenOpActions()
             }
         } else {
             l<<addAction(QIcon::fromTheme("document-open-symbolic"), tr("&Open %1 selected files").arg(m_selections.count()));
-            connect(l.last(), &QAction::triggered, [=](){
+            connect(l.last(), &QAction::triggered, [=]() {
                 qDebug()<<"triggered";
                 QStringList dirs;
                 QStringList files;
@@ -256,6 +282,13 @@ const QList<QAction *> DirectoryViewMenu::constructCreateTemplateActions()
     QList<QAction *> l;
     if (m_selections.isEmpty()) {
         auto createAction = new QAction(tr("&New..."), this);
+        if (m_is_cd) {
+            createAction->setEnabled(false);
+        }
+        if(m_directory.compare(QString::fromLocal8Bit("trash:///")) == 0)
+        {
+            createAction->setEnabled(false);
+        }
         l<<createAction;
         QMenu *subMenu = new QMenu(this);
         createAction->setMenu(subMenu);
@@ -269,8 +302,10 @@ const QList<QAction *> DirectoryViewMenu::constructCreateTemplateActions()
             for (auto t : templates) {
                 QFileInfo info(t);
                 QAction *action = new QAction(p.icon(info), info.baseName(), this);
-                connect(action, &QAction::triggered, [=](){
+                connect(action, &QAction::triggered, [=]() {
                     CreateTemplateOperation op(m_directory, CreateTemplateOperation::Template, t);
+                    FileOperationErrorDialog dlg;
+                    connect(&op, &CreateTemplateOperation::errored, &dlg, &FileOperationErrorDialog::handleError);
                     op.run();
                     auto target = op.target();
                     m_uris_to_edit<<target;
@@ -280,12 +315,14 @@ const QList<QAction *> DirectoryViewMenu::constructCreateTemplateActions()
             subMenu->addSeparator();
         }
 
-         QList<QAction *> actions;
+        QList<QAction *> actions;
         auto createEmptyFileAction = new QAction(QIcon::fromTheme("document-new-symbolic"), tr("Empty &File"), this);
         actions<<createEmptyFileAction;
-        connect(actions.last(), &QAction::triggered, [=](){
+        connect(actions.last(), &QAction::triggered, [=]() {
             //FileOperationUtils::create(m_directory);
             CreateTemplateOperation op(m_directory);
+            FileOperationErrorDialog dlg;
+            connect(&op, &CreateTemplateOperation::errored, &dlg, &FileOperationErrorDialog::handleError);
             op.run();
             auto targetUri = op.target();
             qDebug()<<"target:"<<targetUri;
@@ -293,9 +330,11 @@ const QList<QAction *> DirectoryViewMenu::constructCreateTemplateActions()
         });
         auto createFolderActions = new QAction(QIcon::fromTheme("folder-new-symbolic"), tr("&Folder"), this);
         actions<<createFolderActions;
-        connect(actions.last(), &QAction::triggered, [=](){
+        connect(actions.last(), &QAction::triggered, [=]() {
             //FileOperationUtils::create(m_directory, nullptr, CreateTemplateOperation::EmptyFolder);
             CreateTemplateOperation op(m_directory, CreateTemplateOperation::EmptyFolder, tr("New Folder"));
+            FileOperationErrorDialog dlg;
+            connect(&op, &CreateTemplateOperation::errored, &dlg, &FileOperationErrorDialog::handleError);
             op.run();
             auto targetUri = op.target();
             qDebug()<<"target:"<<targetUri;
@@ -315,7 +354,6 @@ const QList<QAction *> DirectoryViewMenu::constructViewOpActions()
         ViewFactorySortFilterModel2 model;
         model.setDirectoryUri(m_directory);
         auto viewNames = model.supportViewIds();
-
         auto viewFactorysManager = DirectoryViewFactoryManager2::getInstance();
 
         if (!viewNames.isEmpty()) {
@@ -324,13 +362,14 @@ const QList<QAction *> DirectoryViewMenu::constructViewOpActions()
             l<<viewTypeAction;
             QMenu *viewTypeSubMenu = new QMenu(this);
             for (auto viewId : viewNames) {
-                auto viewType = viewTypeSubMenu->addAction(viewFactorysManager->getFactory(viewId)->viewIcon(), viewId);
+                auto viewType = viewTypeSubMenu->addAction(viewFactorysManager->getFactory(viewId)->viewIcon(), viewFactorysManager->getFactory(viewId)->viewName());
+                viewType->setData(viewId);
                 if (m_view->viewId() == viewId) {
                     viewType->setCheckable(true);
                     viewType->setChecked(true);
                 } else {
-                    connect(viewType, &QAction::triggered, [=](){
-                        m_top_window->beginSwitchView(viewType->text());
+                    connect(viewType, &QAction::triggered, [=]() {
+                        m_top_window->beginSwitchView(viewType->data().toString());
                     });
                 }
             }
@@ -344,8 +383,8 @@ const QList<QAction *> DirectoryViewMenu::constructViewOpActions()
 
         QList<QAction *> tmp;
         tmp<<sortTypeMenu->addAction(tr("Name"));
-        tmp<<sortTypeMenu->addAction(tr("File Type"));
         tmp<<sortTypeMenu->addAction(tr("File Size"));
+        tmp<<sortTypeMenu->addAction(tr("File Type"));
         tmp<<sortTypeMenu->addAction(tr("Modified Date"));
         int sortType = m_view->getSortType();
         if (sortType >= 0) {
@@ -354,7 +393,7 @@ const QList<QAction *> DirectoryViewMenu::constructViewOpActions()
         }
 
         for (int i = 0; i < tmp.count(); i++) {
-            connect(tmp.at(i), &QAction::triggered, [=](){
+            connect(tmp.at(i), &QAction::triggered, [=]() {
                 m_top_window->setCurrentSortColumn(i);
             });
         }
@@ -373,7 +412,7 @@ const QList<QAction *> DirectoryViewMenu::constructViewOpActions()
         tmp.at(sortOrder)->setChecked(true);
 
         for (int i = 0; i < tmp.count(); i++) {
-            connect(tmp.at(i), &QAction::triggered, [=](){
+            connect(tmp.at(i), &QAction::triggered, [=]() {
                 m_top_window->setCurrentSortOrder(Qt::SortOrder(i));
             });
         }
@@ -387,24 +426,28 @@ const QList<QAction *> DirectoryViewMenu::constructViewOpActions()
         auto folderFirst = sortPreferencesMenu->addAction(tr("Folder First"));
         folderFirst->setCheckable(true);
         folderFirst->setChecked(m_top_window->getWindowSortFolderFirst());
-        connect(folderFirst, &QAction::triggered, this, [=](bool checked){
-            m_top_window->setSortFolderFirst(checked);
+        connect(folderFirst, &QAction::triggered, this, [=]() {
+            m_top_window->setSortFolderFirst();
+            folderFirst->setChecked(m_top_window->getWindowSortFolderFirst());
         });
 
         if (QLocale::system().name().contains("zh")) {
             auto useDefaultNameSortOrder = sortPreferencesMenu->addAction(tr("Chinese First"));
             useDefaultNameSortOrder->setCheckable(true);
-            useDefaultNameSortOrder->setChecked(!m_top_window->getWindowUseDefaultNameSortOrder());
-            connect(useDefaultNameSortOrder, &QAction::triggered, this, [=](bool checked){
-                m_top_window->setUseDefaultNameSortOrder(!checked);
+            useDefaultNameSortOrder->setChecked(m_top_window->getWindowUseDefaultNameSortOrder());
+            connect(useDefaultNameSortOrder, &QAction::triggered, this, [=]() {
+                m_top_window->setUseDefaultNameSortOrder();
+                bool checked = m_top_window->getWindowUseDefaultNameSortOrder();
+                useDefaultNameSortOrder->setChecked(checked);
             });
         }
 
         auto showHidden = sortPreferencesMenu->addAction(tr("Show Hidden"));
         showHidden->setCheckable(true);
         showHidden->setChecked(m_top_window->getWindowShowHidden());
-        connect(showHidden, &QAction::triggered, this, [=](bool checked){
-            m_top_window->setShowHidden(checked);
+        connect(showHidden, &QAction::triggered, this, [=]() {
+            m_top_window->setShowHidden();
+            showHidden->setChecked(m_top_window->getWindowShowHidden());
         });
 
         sortPreferencesAction->setMenu(sortPreferencesMenu);
@@ -418,22 +461,28 @@ const QList<QAction *> DirectoryViewMenu::constructFileOpActions()
     QList<QAction *> l;
 
     if (!m_is_trash && !m_is_search && !m_is_computer) {
-        if (!m_selections.isEmpty()) {
+        QString homeUri = "file://" +  QStandardPaths::writableLocation(QStandardPaths::HomeLocation);
+        if (!m_selections.isEmpty() && !m_selections.contains(homeUri)) {
             l<<addAction(QIcon::fromTheme("edit-copy-symbolic"), tr("&Copy"));
-            connect(l.last(), &QAction::triggered, [=](){
+            connect(l.last(), &QAction::triggered, [=]() {
                 ClipboardUtils::setClipboardFiles(m_selections, false);
             });
             l<<addAction(QIcon::fromTheme("edit-cut-symbolic"), tr("Cu&t"));
-            connect(l.last(), &QAction::triggered, [=](){
+            connect(l.last(), &QAction::triggered, [=]() {
                 ClipboardUtils::setClipboardFiles(m_selections, true);
             });
-            l<<addAction(QIcon::fromTheme("edit-delete-symbolic"), tr("&Delete"));
-            connect(l.last(), &QAction::triggered, [=](){
+            l<<addAction(QIcon::fromTheme("edit-delete-symbolic"), tr("&Delete to trash"));
+            connect(l.last(), &QAction::triggered, [=]() {
                 FileOperationUtils::trash(m_selections, true);
             });
+            //add delete forever option
+            l<<addAction(QIcon::fromTheme("edit-clear-symbolic"), tr("Delete forever"));
+            connect(l.last(), &QAction::triggered, [=]() {
+                FileOperationUtils::executeRemoveActionWithDialog(m_selections);
+            });
             if (m_selections.count() == 1) {
                 l<<addAction(QIcon::fromTheme("document-edit-symbolic"), tr("&Rename"));
-                connect(l.last(), &QAction::triggered, [=](){
+                connect(l.last(), &QAction::triggered, [=]() {
                     m_view->editUri(m_selections.first());
                 });
             }
@@ -441,16 +490,34 @@ const QList<QAction *> DirectoryViewMenu::constructFileOpActions()
             auto pasteAction = addAction(QIcon::fromTheme("edit-paste-symbolic"), tr("&Paste"));
             l<<pasteAction;
             pasteAction->setEnabled(ClipboardUtils::isClipboardHasFiles());
-            connect(l.last(), &QAction::triggered, [=](){
+            connect(l.last(), &QAction::triggered, [=]() {
                 ClipboardUtils::pasteClipboardFiles(m_directory);
             });
             l<<addAction(QIcon::fromTheme("view-refresh-symbolic"), tr("&Refresh"));
-            connect(l.last(), &QAction::triggered, [=](){
+            connect(l.last(), &QAction::triggered, [=]() {
                 m_top_window->refresh();
             });
         }
     }
 
+    //select all and reverse select
+    if (m_selections.isEmpty())
+    {
+        l<<addAction(tr("Select &All"));
+        connect(l.last(), &QAction::triggered, [=]() {
+            //qDebug() << "select all";
+            m_view->invertSelections();
+        });
+    }
+    else
+    {
+        l<<addAction(tr("Reverse Select"));
+        connect(l.last(), &QAction::triggered, [=]() {
+            //qDebug() << "Reverse select";
+            m_view->invertSelections();
+        });
+    }
+
     return l;
 }
 
@@ -459,8 +526,8 @@ const QList<QAction *> DirectoryViewMenu::constructFilePropertiesActions()
     QList<QAction *> l;
 
     if (!m_is_search) {
-        l<<addAction(QIcon::fromTheme("preview-file"), tr("&Properties"));
-        connect(l.last(), &QAction::triggered, [=](){
+        l<<addAction(QIcon::fromTheme("preview-file"), tr("P&roperties"));
+        connect(l.last(), &QAction::triggered, [=]() {
             //FIXME:
             if (m_selections.isEmpty()) {
                 QStringList uris;
@@ -475,8 +542,8 @@ const QList<QAction *> DirectoryViewMenu::constructFilePropertiesActions()
             }
         });
     } else if (m_selections.count() == 1) {
-        l<<addAction(QIcon::fromTheme("preview-file"), tr("&Properties"));
-        connect(l.last(), &QAction::triggered, [=](){
+        l<<addAction(QIcon::fromTheme("preview-file"), tr("P&roperties"));
+        connect(l.last(), &QAction::triggered, [=]() {
             PropertiesWindow *p = new PropertiesWindow(m_selections);
             p->setAttribute(Qt::WA_DeleteOnClose);
             p->show();
@@ -521,10 +588,10 @@ const QList<QAction *> DirectoryViewMenu::constructTrashActions()
         if (m_selections.isEmpty()) {
             l<<addAction(QIcon::fromTheme("window-close-symbolic"), tr("&Clean the Trash"));
             l.last()->setEnabled(!isTrashEmpty);
-            connect(l.last(), &QAction::triggered, [=](){
+            connect(l.last(), &QAction::triggered, [=]() {
                 auto result = QMessageBox::question(nullptr, tr("Delete Permanently"), tr("Are you sure that you want to delete these files? "
-                                                                                          "Once you start a deletion, the files deleting will never be "
-                                                                                          "restored again."));
+                                                    "Once you start a deletion, the files deleting will never be "
+                                                    "restored again."));
                 if (result == QMessageBox::Yes) {
                     auto uris = m_top_window->getCurrentAllFileUris();
                     FileOperationUtils::remove(uris);
@@ -532,7 +599,7 @@ const QList<QAction *> DirectoryViewMenu::constructTrashActions()
             });
         } else {
             l<<addAction(tr("&Restore"));
-            connect(l.last(), &QAction::triggered, [=](){
+            connect(l.last(), &QAction::triggered, [=]() {
                 if (m_selections.count() == 1) {
                     FileOperationUtils::restore(m_selections.first());
                 } else {
@@ -540,10 +607,10 @@ const QList<QAction *> DirectoryViewMenu::constructTrashActions()
                 }
             });
             l<<addAction(QIcon::fromTheme("window-close-symbolic"), tr("&Delete"));
-            connect(l.last(), &QAction::triggered, [=](){
+            connect(l.last(), &QAction::triggered, [=]() {
                 auto result = QMessageBox::question(nullptr, tr("Delete Permanently"), tr("Are you sure that you want to delete these files? "
-                                                                                          "Once you start a deletion, the files deleting will never be "
-                                                                                          "restored again."));
+                                                    "Once you start a deletion, the files deleting will never be "
+                                                    "restored again."));
                 if (result == QMessageBox::Yes) {
                     FileOperationUtils::remove(m_selections);
                 }
@@ -562,21 +629,22 @@ const QList<QAction *> DirectoryViewMenu::constructSearchActions()
             return l;
 
         l<<addAction(QIcon::fromTheme("new-window-symbolc"), tr("Open Parent Folder in New Window"));
-        connect(l.last(), &QAction::triggered, [=](){
+        connect(l.last(), &QAction::triggered, [=]() {
             for (auto uri : m_selections) {
                 auto parentUri = FileUtils::getParentUri(uri);
                 if (!parentUri.isNull()) {
-                    FMWindow *newWindow = new FMWindow(parentUri);
+                    auto *windowIface = m_top_window->create(parentUri);
+                    auto newWindow = dynamic_cast<QWidget *>(windowIface);
                     auto selection = m_selections;
 #if QT_VERSION > QT_VERSION_CHECK(5, 12, 0)
-                    QTimer::singleShot(1000, newWindow, [=](){
+                    QTimer::singleShot(1000, newWindow, [=]() {
                         if (newWindow)
-                            newWindow->setCurrentSelectionUris(selection);
+                            windowIface->setCurrentSelectionUris(selection);
                     });
 #else
-                    QTimer::singleShot(1000, [=](){
+                    QTimer::singleShot(1000, [=]() {
                         if (newWindow)
-                            newWindow->setCurrentSelectionUris(selection);
+                            windowIface->setCurrentSelectionUris(selection);
                     });
 #endif
                     newWindow->show();
@@ -591,6 +659,9 @@ const QList<QAction *> DirectoryViewMenu::constructMenuPluginActions()
 {
     QList<QAction *> l;
     auto pluginIds = MenuPluginManager::getInstance()->getPluginIds();
+    //sort plugiins by name, so the menu option orders is relatively fixed
+    qSort(pluginIds.begin(), pluginIds.end());
+
     for (auto id : pluginIds) {
         auto plugin = MenuPluginManager::getInstance()->getPlugin(id);
         auto actions = plugin->menuActions(MenuPluginInterface::DirectoryView, m_directory, m_selections);
